/*
 * 	Copyright (C) 2012, Pierre Letessier (INA / INRIA), all rights reserved.
 *
 * 	This file is part of BelgaLogosEval.
 *
 *   BelgaLogosEval is free software: you can redistribute it and/or modify
 *  	it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation, either version 3 of the License, or
 *   (at your option) any later version.
 *
 *   BelgaLogosEval is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with BelgaLogosEval.  If not, see <http://www.gnu.org/licenses/>.
 */

#include <stdio.h>
#include <stdlib.h>

#ifdef _MSV_VER
	#if _MSV_VER < 1600
		typedef __int16				int16_t;
		typedef unsigned __int16	uint16_t;
		typedef __int32				int32_t;
		typedef unsigned __int32	uint32_t;
		typedef __int64				int64_t;
		typedef unsigned __int64	uint64_t;
	#else
		#include <stdint.h>
	#endif
#else
	#include <stdint.h>
#endif

#include <string.h>
#include <fstream>
#include <iostream>
#include <sstream>
#include <string.h>
#include <iomanip>

#include <vector>
#include <map>
#include <set>
#include <algorithm>
#include "Result.h"
#include "ResultManager.h"
#include "Groundtruth.h"

using namespace std;

typedef struct
{
	float f_score;
	bool b_good;
	float f_precision;
	float f_recall;
} DetectionScore;

struct cmp_str
{
	bool operator()(const char *a, const char *b) const
	{
		return strcmp(a, b) < 0;
	}
};

bool compfunctionForDS(DetectionScore const& a, DetectionScore const& b)
{
	return (a.f_score > b.f_score);
}

struct pkt_less
{
	bool operator ()(DetectionScore const& a, DetectionScore const& b) const
	{
		if (a.f_score < b.f_score)
			return true;
		else
			return false;
	}
};

int main(int argc, char* argv[])
{
	if (argc != 3)
	{
		printf(
				"USAGE : InstanceSearchEval.exe s_groundtruth_filename s_result_filename\n");
		exit(1);
	}

	char* s_groundtruth_filename = argv[1];
	char* s_result_filename = argv[2];

	Groundtruth gt(s_groundtruth_filename);

	ResultManager irm;
	irm.readResultsFromFile(s_result_filename);

	map<char*, vector<DetectionScore>*, cmp_str> a_detection_scores_per_instance;
	map<char*, char*, cmp_str> a_class_for_instance;
	map<char*, float, cmp_str> a_f_map_for_class;

	uint32_t ui32_res_nb = irm.getNumberOfResult();

	for (uint32_t ui32_i = 0; ui32_i < ui32_res_nb; ui32_i++)
	{
		Result ir = irm.getResult(ui32_i);

		float f_intersection = -999.0f;
		int32_t i32_detection_type = gt.findResultInGroundtruth(
				ir.getClassName(), ir.getResultFilename(), ir.getX1(),
				ir.getY1(), ir.getX2(), ir.getY2(), &f_intersection);

		if (i32_detection_type != 0)
		{
			DetectionScore ds;
			ds.f_score = ir.getScore();
			ds.b_good = false;

			if (f_intersection > 0.02f)
				if (i32_detection_type == 1)
					ds.b_good = true;

			map<char*, vector<DetectionScore>*, cmp_str>::const_iterator itr_find =
					a_detection_scores_per_instance.find(ir.getQueryName());
			if (itr_find == a_detection_scores_per_instance.end())
				a_detection_scores_per_instance[ir.getQueryName()] = new vector<
						DetectionScore>;

			a_detection_scores_per_instance[ir.getQueryName()]->push_back(ds);

			a_class_for_instance[ir.getQueryName()] = ir.getClassName();
		}

		if ((ui32_i % (ui32_res_nb / 100)) == 0)
			cout << "\rEvaluating the results: " << ui32_i << " / "
					<< ui32_res_nb << flush;
	}

	cout << endl << endl;

	for (map<char*, char*, cmp_str>::const_iterator itr = a_class_for_instance.begin();
			itr != a_class_for_instance.end(); ++itr)
	{
		a_f_map_for_class[(*itr).second] = 0;
	}

	double f_map_global = 0.0;
	uint32_t ui32_nb_good_instances = 0;

	for (map<char*, vector<DetectionScore>*, cmp_str>::const_iterator itr =
			a_detection_scores_per_instance.begin();
			itr != a_detection_scores_per_instance.end(); ++itr)
	{
		//cout << (*itr).first << endl;

		vector<DetectionScore>* p_vec = (*itr).second;

		uint32_t ui32_nb_good_detection = 0;
		uint32_t ui32_nb_bad_detection = 0;

		double f_map = 0;

		sort(p_vec->begin(), p_vec->end(), &compfunctionForDS);

		for (vector<DetectionScore>::iterator it_ds = (*itr).second->begin();
				it_ds != (*itr).second->end(); ++it_ds)
		{
			if ((*it_ds).b_good == 1)
				ui32_nb_good_detection++;

			if ((*it_ds).b_good == 0)
				ui32_nb_bad_detection++;

			(*it_ds).f_precision = (float) ui32_nb_good_detection
					/ (float) (ui32_nb_good_detection + ui32_nb_bad_detection);

			if ((*it_ds).b_good == 1)
				f_map += (*it_ds).f_precision;

			(*it_ds).f_recall = (float) ui32_nb_good_detection
					/ (float) (gt.getNumberOfInstancesInThisClass(
							a_class_for_instance[(*itr).first]));

			if ((*it_ds).f_recall > 0.99999999f)
			{
				break;
			}

		}

		f_map_global += f_map;

		f_map /= gt.getNumberOfInstancesInThisClass(a_class_for_instance[(*itr).first]);
		a_f_map_for_class[a_class_for_instance[(*itr).first]] += f_map;

		ui32_nb_good_instances += gt.getNumberOfInstancesInThisClass(
				a_class_for_instance[(*itr).first]);

		//cout << "f_map = " << f_map << endl;
	}

	cout << right << setw(17)
					<< "Logo name  "
					<< left
					<< "Mean Average Precision"
					<< endl;

	for (map<char*, float, cmp_str>::const_iterator itr = a_f_map_for_class.begin();
			itr != a_f_map_for_class.end(); ++itr)
	{
		cout << right << setw(17)
				<< (*itr).first
				<< " "
				<< left
				<< ((*itr).second)
						/ gt.getNumberOfInstancesInThisClass((*itr).first)
				<< endl;
	}

	f_map_global /= ui32_nb_good_instances;
	cout << endl << "Global Mean Average Precision = " << f_map_global << endl;

	return EXIT_SUCCESS;
}
