/*
* 	Copyright (C) 2012, Pierre Letessier (INA / INRIA), all rights reserved.
*
* 	This file is part of BelgaLogosEval.
*
*   BelgaLogosEval is free software: you can redistribute it and/or modify
*  	it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   BelgaLogosEval is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with BelgaLogosEval.  If not, see <http://www.gnu.org/licenses/>.
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fstream>
#include <iostream>
#include <sstream>
#include <vector>
#include "Groundtruth.h"

using namespace std;

#define MAX(a,b) (a>b?a:b)
#define MIN(a,b) (a<b?a:b)

GTClass::GTClass(char* s_name)
{
	this->s_name = (char*) malloc(strlen(s_name) + 1);
	sprintf(this->s_name, "%s", s_name);
}

float GTClass::computeIntersectionSize(uint32_t ui32_x1, uint32_t ui32_y1,
		uint32_t ui32_x2, uint32_t ui32_y2, uint32_t ui32_x3, uint32_t ui32_y3,
		uint32_t ui32_x4, uint32_t ui32_y4)
{
	uint32_t ui32_b_xmin = MAX(ui32_x1,ui32_x3);
	uint32_t ui32_b_xmax = MIN(ui32_x2,ui32_x4);
	uint32_t ui32_b_ymin = MAX(ui32_y1,ui32_y3);
	uint32_t ui32_b_ymax = MIN(ui32_y2,ui32_y4);

	uint32_t ui32_width = 0;
	if (ui32_b_xmin < ui32_b_xmax)
		ui32_width = ui32_b_xmax - ui32_b_xmin;

	uint32_t ui32_height = 0;
	if (ui32_b_ymin < ui32_b_ymax)
		ui32_height = ui32_b_ymax - ui32_b_ymin;

	uint32_t ui32_size = ui32_width * ui32_height;

	float f_size = ((float) ui32_size)
			/ ((float) ((ui32_x2 - ui32_x1) * (ui32_y2 - ui32_y1)));

	return f_size;
}

bool GTClass::compareResultWithGT(char* s_filename, uint32_t ui32_x1,
		uint32_t ui32_y1, uint32_t ui32_x2, uint32_t ui32_y2,
		float* p_f_intersection)
{
	bool b_intersection = false;
	float f_intersection_max = 0.0f;

	for (set<Instance, compInstance>::iterator it = a_instances.begin();
			it != a_instances.end(); it++)
	{
		if (strcmp((*it).s_filename, s_filename) == 0)
		{
			float f_intersection = computeIntersectionSize((*it).ui32_x1,
					(*it).ui32_y1, (*it).ui32_x2, (*it).ui32_y2, ui32_x1,
					ui32_y1, ui32_x2, ui32_y2);
			if (f_intersection > f_intersection_max)
			{
				f_intersection_max = f_intersection;
				b_intersection = true;
			}
		}
	}

	if (b_intersection)
		*p_f_intersection = f_intersection_max;
	else
		*p_f_intersection = 0.0f;

	return b_intersection;
}

void GTClass::addNewInstance(Instance* p_instance)
{
	a_instances.insert(*p_instance);
}

GTClass* GroupOfGTClasses::findGTClass(char* s_class_name)
{
	GTClass gtclass(s_class_name);

	set<GTClass, compGTClass>::iterator it = a_gt_classes.find(gtclass);

	if (it != a_gt_classes.end())
		return const_cast<GTClass*>(&*it);
	else
		return NULL;
}

GTClass* GroupOfGTClasses::addNewGTClass(char* s_class_name)
{
	GTClass* gtclass = new GTClass(s_class_name);
	if (gtclass == NULL)
	{
		exit(1);
	}

	a_gt_classes.insert(*gtclass);

	return gtclass;
}

Groundtruth::Groundtruth(char* s_gt_filename)
{

	ui32_number_of_gt_classes_ = 0;
	ui32_number_of_instances_ = 0;

	char s_name[512];
	char s_class[512];
	char s_filename[512];
	char s_type[512];
	GTType_t i_type;
	uint32_t ui32_state;
	GTState_t i_state;
	uint32_t ui32_x1;
	uint32_t ui32_y1;
	uint32_t ui32_x2;
	uint32_t ui32_y2;

	ifstream fin;
	fin.open(s_gt_filename);

	string input;
	bool error = false;

	while (true)
	{
		getline(fin, input);

		if (!fin)
			break;

		istringstream buffer(input);
		buffer >> s_name >> s_class >> s_filename >> s_type >> ui32_state
				>> ui32_x1 >> ui32_y1 >> ui32_x2 >> ui32_y2;

		if (!buffer || !buffer.eof())
		{
			error = true;
			break;
		}

		if (strcmp(s_type, "face") == 0)
			i_type = GTType_FACE;
		else
			i_type = GTType_LOGO;

		if (ui32_state == 1)
			i_state = GTState_accepted;
		else
			i_state = GTState_rejected;

		Instance* p_instance = (Instance*) malloc(sizeof(Instance));
		if (p_instance == NULL)
		{
			error = true;
			break;
		}

		p_instance->s_name = (char*) malloc(strlen(s_name) + 1);
		sprintf(p_instance->s_name, "%s", s_name);

		p_instance->s_class = (char*) malloc(strlen(s_class) + 1);
		sprintf(p_instance->s_class, "%s", s_class);

		p_instance->s_filename = (char*) malloc(strlen(s_filename) + 1);
		sprintf(p_instance->s_filename, "%s", s_filename);

		p_instance->i_type = i_type;
		p_instance->i_state = i_state;
		p_instance->ui32_x1 = ui32_x1;
		p_instance->ui32_y1 = ui32_y1;
		p_instance->ui32_x2 = ui32_x2;
		p_instance->ui32_y2 = ui32_y2;

		GroupOfGTClasses* p_grp_gt_classes = NULL;

		if (p_instance->i_type == GTType_FACE)
		{
			if (p_instance->i_state == GTState_accepted)
				p_grp_gt_classes = &accepted_faces_;
			else if (p_instance->i_state == GTState_rejected)
				p_grp_gt_classes = &rejected_faces_;
		}
		else if (p_instance->i_type == GTType_LOGO)
		{
			if (p_instance->i_state == GTState_accepted)
				p_grp_gt_classes = &accepted_logos_;
			else if (p_instance->i_state == GTState_rejected)
				p_grp_gt_classes = &rejected_logos_;
		}

		GTClass* p_gtclass = p_grp_gt_classes->findGTClass(p_instance->s_class);

		if (p_gtclass == NULL)
		{
			p_gtclass = p_grp_gt_classes->addNewGTClass(p_instance->s_class);
			ui32_number_of_gt_classes_++;
		}

		p_gtclass->addNewInstance(p_instance);
		ui32_number_of_instances_++;

		if (p_instance->i_state == GTState_accepted)
			a_number_of_good_instances_per_class_[p_instance->s_class]++;
	}

	if (error)
		cout << "There is an error" << endl;

	cout << "The groundtruth has been read" << endl;

}

Groundtruth::~Groundtruth()
{

}

int32_t Groundtruth::findResultInGroundtruth(char* s_class_name,
		char* s_filename, uint32_t ui32_x1, uint32_t ui32_y1, uint32_t ui32_x2,
		uint32_t ui32_y2, float* p_f_intersection)
{
	bool b_found_a_result = false;

	GTClass* p_gt_class = accepted_faces_.findGTClass(s_class_name);
	if (p_gt_class != NULL)
	{
		b_found_a_result = p_gt_class->compareResultWithGT(s_filename, ui32_x1,
				ui32_y1, ui32_x2, ui32_y2, p_f_intersection);
	}

	if (b_found_a_result == true)
	{
		return 1;
	}

	p_gt_class = accepted_logos_.findGTClass(s_class_name);
	if (p_gt_class != NULL)
	{
		b_found_a_result = p_gt_class->compareResultWithGT(s_filename, ui32_x1,
				ui32_y1, ui32_x2, ui32_y2, p_f_intersection);
	}

	if (b_found_a_result == true)
	{
		return 1;
	}

	p_gt_class = rejected_faces_.findGTClass(s_class_name);
	if (p_gt_class != NULL)
		b_found_a_result = p_gt_class->compareResultWithGT(s_filename, ui32_x1,
				ui32_y1, ui32_x2, ui32_y2, p_f_intersection);

	if (b_found_a_result == true)
	{
		return 0;
	}

	p_gt_class = rejected_logos_.findGTClass(s_class_name);
	if (p_gt_class != NULL)
		b_found_a_result = p_gt_class->compareResultWithGT(s_filename, ui32_x1,
				ui32_y1, ui32_x2, ui32_y2, p_f_intersection);

	if (b_found_a_result == true)
	{
		return 0;
	}

	return -1;
}

uint32_t Groundtruth::getNumberOfGTClasses()
{
	return ui32_number_of_gt_classes_;
}

uint32_t Groundtruth::getNumberOfInstances()
{
	return ui32_number_of_instances_;
}

uint32_t Groundtruth::getNumberOfInstancesInThisClass(char* s_class_name)
{
	return a_number_of_good_instances_per_class_[s_class_name];
}

Instance* Groundtruth::copyInstance(Instance* p_instance)
{
	Instance* p_copy = (Instance*) malloc(sizeof(Instance));

	p_copy->s_name = (char*) malloc(strlen(p_instance->s_name) + 1);
	sprintf(p_copy->s_name, "%s", p_instance->s_name);

	p_copy->s_filename = (char*) malloc(strlen(p_instance->s_filename) + 1);
	sprintf(p_copy->s_filename, "%s", p_instance->s_filename);

	p_copy->s_class = (char*) malloc(strlen(p_instance->s_class) + 1);
	sprintf(p_copy->s_class, "%s", p_instance->s_class);

	p_copy->i_type = p_instance->i_type;
	p_copy->i_state = p_instance->i_state;
	p_copy->ui32_x1 = p_instance->ui32_x1;
	p_copy->ui32_y1 = p_instance->ui32_y1;
	p_copy->ui32_x2 = p_instance->ui32_x2;
	p_copy->ui32_y2 = p_instance->ui32_y2;

	return p_copy;
}

Instance* Groundtruth::getAFalseLogoInstance()
{
	Instance* p_instance = (Instance*) malloc(sizeof(Instance));

	p_instance->s_name = "VRT0001";
	p_instance->s_filename = "image0001.jpg";
	p_instance->s_class = "VRT";
	p_instance->i_type = GTType_LOGO;
	p_instance->i_state = GTState_accepted;
	p_instance->ui32_x1 = 10000;
	p_instance->ui32_y1 = 10000;
	p_instance->ui32_x2 = 10010;
	p_instance->ui32_y2 = 10010;

	return p_instance;
}

Instance* Groundtruth::getRandomlyALogoInstanceWithErrors()
{
	Instance* p_instance = copyInstance(getRandomlyALogoInstance());

	uint32_t ui32_x1 = p_instance->ui32_x1;
	uint32_t ui32_y1 = p_instance->ui32_y1;
	uint32_t ui32_x2 = p_instance->ui32_x2;
	uint32_t ui32_y2 = p_instance->ui32_y2;

	uint32_t ui32_width = ui32_x2 - ui32_x1;
	uint32_t ui32_height = ui32_y2 - ui32_y1;

	int i_rand = (rand() % (ui32_width / 2)) - (ui32_width / 4);
	ui32_x1 += i_rand;

	i_rand = (rand() % (ui32_width / 2)) - (ui32_width / 4);
	ui32_x2 += i_rand;

	i_rand = (rand() % (ui32_height / 2)) - (ui32_height / 4);
	ui32_y1 += i_rand;

	i_rand = (rand() % (ui32_height / 2)) - (ui32_height / 4);
	ui32_y2 += i_rand;

	return p_instance;
}

Instance* Groundtruth::getRandomlyALogoInstance()
{
	int i_rand;

	i_rand = rand() % 2;

	if (i_rand == 0)
	{
		i_rand = rand() % (accepted_logos_.a_gt_classes.size());

		int i_index = 0;
		for (set<GTClass, compGTClass>::iterator it = accepted_logos_.a_gt_classes.begin();
				it != accepted_logos_.a_gt_classes.end(); it++)
		{
			if (i_index == i_rand)
			{
				i_rand = rand() % ((*it).a_instances.size());

				i_index = 0;
				for (set<Instance, compInstance>::iterator it_i = (*it).a_instances.begin();
						it_i != (*it).a_instances.end(); it_i++)
				{
					if (i_index == i_rand)
					{
						return const_cast<Instance*>(&*it_i);
					}
					i_index++;
				}
			}
			i_index++;
		}
	}
	else
	{
		i_rand = rand() % (rejected_logos_.a_gt_classes.size());

		int i_index = 0;
		for (set<GTClass, compGTClass>::iterator it = rejected_logos_.a_gt_classes.begin();
				it != rejected_logos_.a_gt_classes.end(); it++)
		{
			if (i_index == i_rand)
			{
				i_rand = rand() % ((*it).a_instances.size());

				i_index = 0;
				for (set<Instance, compInstance>::iterator it_i = (*it).a_instances.begin();
						it_i != (*it).a_instances.end(); it_i++)
				{
					if (i_index == i_rand)
					{
						return const_cast<Instance*>(&*it_i);
					}
					i_index++;
				}
			}
			i_index++;
		}
	}
	return NULL;
}
