/*
* 	Copyright (C) 2012, Pierre Letessier (INA / INRIA), all rights reserved.
*
* 	This file is part of BelgaLogosEval.
*
*   BelgaLogosEval is free software: you can redistribute it and/or modify
*  	it under the terms of the GNU General Public License as published by
*   the Free Software Foundation, either version 3 of the License, or
*   (at your option) any later version.
*
*   BelgaLogosEval is distributed in the hope that it will be useful,
*   but WITHOUT ANY WARRANTY; without even the implied warranty of
*   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
*   GNU General Public License for more details.
*
*   You should have received a copy of the GNU General Public License
*   along with BelgaLogosEval.  If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef GROUNDTRUTH_H_
#define GROUNDTRUTH_H_

#ifdef _MSV_VER
	#if _MSV_VER < 1600
		typedef __int16				int16_t;
		typedef unsigned __int16	uint16_t;
		typedef __int32				int32_t;
		typedef unsigned __int32	uint32_t;
		typedef __int64				int64_t;
		typedef unsigned __int64	uint64_t;
	#else
		#include <stdint.h>
	#endif
#else
	#include <stdint.h>
#endif

#include <set>
#include <map>
#include <string.h>

enum GTType_t
{
	GTType_FACE, GTType_LOGO
};
enum GTState_t
{
	GTState_accepted, GTState_rejected
};

typedef struct
{
	char* s_name;
	char* s_class;
	char* s_filename;
	GTType_t i_type;
	GTState_t i_state;
	uint32_t ui32_x1;
	uint32_t ui32_y1;
	uint32_t ui32_x2;
	uint32_t ui32_y2;
} Instance;

struct compInstance
{
	bool operator()(const Instance& a, const Instance& b) const
	{
		return strcmp(a.s_name, b.s_name) < 0;
	}
};

class GTClass
{
public:
	GTClass(char* s_name);

	float computeIntersectionSize(uint32_t ui32_x1, uint32_t ui32_y1,
			uint32_t ui32_x2, uint32_t ui32_y2, uint32_t ui32_x3,
			uint32_t ui32_y3, uint32_t ui32_x4, uint32_t ui32_y4);
	bool compareResultWithGT(char* s_filename, uint32_t ui32_x1,
			uint32_t ui32_y1, uint32_t ui32_x2, uint32_t ui32_y2,
			float* p_f_intersection);

	void addNewInstance(Instance* p_instance);

	char* s_name;
	std::set<Instance, compInstance> a_instances;

};

struct compGTClass
{
	bool operator()(const GTClass& a, const GTClass& b) const
	{
		return strcmp(a.s_name, b.s_name) < 0;
	}
};

class GroupOfGTClasses
{

public:
	GTClass* findGTClass(char* s_class_name);
	GTClass* addNewGTClass(char* s_class_name);

	std::set<GTClass, compGTClass> a_gt_classes;
};

class Groundtruth
{

	struct cmp_str
	{
		bool operator()(const char* a, const char* b) const
		{	
			return strcmp(a, b) < 0;
		}
	};

public:
	Groundtruth(char* s_filename);
	~Groundtruth();

	uint32_t getNumberOfGTClasses();
	uint32_t getNumberOfInstances();
	uint32_t getNumberOfInstancesInThisClass(char* s_class_name);

	Instance* getAFalseLogoInstance();
	Instance* getRandomlyALogoInstanceWithErrors();
	Instance* getRandomlyALogoInstance();

	Instance* copyInstance(Instance* p_instance);

	int32_t findResultInGroundtruth(char* s_class_name, char* s_filename,
			uint32_t ui32_x1, uint32_t ui32_y1, uint32_t ui32_x2,
			uint32_t ui32_y2, float* p_f_intersection);

private:
	GroupOfGTClasses accepted_faces_;
	GroupOfGTClasses rejected_faces_;
	GroupOfGTClasses accepted_logos_;
	GroupOfGTClasses rejected_logos_;

	uint32_t ui32_number_of_gt_classes_;
	uint32_t ui32_number_of_instances_;

	std::map<char*, uint32_t, cmp_str> a_number_of_good_instances_per_class_;

};

#endif /* GROUNDTRUTH_H_ */
